#!perl

use strict;
use warnings;

use Test::More;
use Test::Number::Delta;

use File::Temp;
use File::Basename;
use IO::File;
use YAML::Tiny qw[ LoadFile ];

BEGIN { use_ok( 'SAOsac::gi' ) }

cmpgi( $_ ) foreach <data/*.gi>;

sub cmpgi
{
  my ( $gi ) = @_;

  ( my $yaml = $gi ) =~ s/.gi/.yaml/;

  my ( $system, $paraboloid, $hyperboloid ) = LoadFile( $yaml );

  open( GI, $gi ) or die( "couldn't open $gi\n" );
  my $comment = <GI>;
  close( GI );
  chomp $comment;

  my $cmpfile = File::Temp->new;

  SAOsac::gi::create( $cmpfile->filename, $comment,
		      $system, $paraboloid, $hyperboloid );

  my @fid = slurp( $gi );
  my @new = slurp( $cmpfile->filename );

  subtest $gi => sub {

      is( scalar @new, scalar @fid, "number of lines in file" )
	  or plan skip_all => "cannot continue";

      is ( shift( @new), shift( @fid), "comment" );
      is ( pop( @new), pop( @fid), "last line: EOF" );


      # the order of the rest doesn't matter, and since
      # it might not be the same (hashes are involved)
      # just sort them.  alphabetic is ok.

      my @fid = sort @fid;
      my @new = sort @new;

      while ( @fid ) {

	  my ( $fid, $new ) = ( shift(@fid), shift(@new) );

	  my @fid_flds = split( /,|\s+/, $fid, 4 );
	  my @new_flds = split( /,|\s+/, $new, 4 );

	  subtest "$new" => sub { 
	  foreach ( 1..3 ) {
	      delta_ok( $new_flds[$_-1], $fid_flds[$_-1], "field $_" );
	  }

	  is( $new_flds[-1], $fid_flds[-1], "description" );

}

      }

  }

}

sub slurp {

    my $file = shift;

    my $fh = IO::File->new( $file, 'r' )
	or die( "error opening $file\n" );

    return $fh->getlines;

}

done_testing;
